def trun(cmd, *args):
    '''run a command in a thread,
        return a handle on the thread.

    It is a good idea to set up some
    kind of cooperative way to tell
    the thread to exit.

    This function creates a sets a flag
    which your thread can check to see
    when it should stop.

    Alternatively, if the thread has gone
    out of control, stop it with

        t.terminate()

    Be prepared for things to go sideways.

    Pressing Ctrl-C should stop most threads
    cleanly and will .terminate() others.
    '''

    import threading
    if not hasattr(threading, 'pflags'):
        threading.pflags = {}

    ts = list(threading.pflags.keys())
    for t in ts:
        if not t.is_alive():
            del threading.pflags[t]

    t = Thread(target=cmd, args=args)
    threading.pflags[t] = False
    t.start()
    return t

def chkflag(t=None):
    '''return True if thread t has been asked to stop.

    Check current thread if no thread t supplied.
    '''

    import threading
    if t is None:
        t = threading.current_thread()
    return threading.pflags.get(t, False)

def should_stop(t=None):
    'alternate, maybe easier to remember name.'
    return chkflag(t)

def rmflag(t=None):
    import threading
    if t is None:
        t = threading.current_thread()
    if t in threading.pflags:
        del threading.pflags[t]
