#!/usr/bin/env python3
# Copyright 2010-2018 Lee Harr
#
# This file is part of pynguin.
# https://bitbucket.org/leeharr/pynguin/
#
# Pynguin is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pynguin is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pynguin.  If not, see <http://www.gnu.org/licenses/>.


import sys
import os


def setup_devel():
    # Change current dir to where this file lives
    filepath = os.path.realpath(__file__)
    filedir, _ = os.path.split(filepath)
    os.chdir(filedir)

    # Use the pynguin lib here in the lib folder.
    # For development, or running from the source
    #   folder instead of a proper install.
    sys.path.insert(0, 'lib')

def _md_file_rewrite(fpth, rules=None, append=None):
    lines = open(fpth).readlines()
    f = open(fpth, 'w')
    for line in lines:
        line = line.rstrip()
        if rules is not None:
            for rule in rules:
                line = rule(line)
        f.write(line)
        f.write('\n')

    if append is not None:
        for line in append:
            f.write(line)
            f.write('\n')

    f.close()

def _md_line_replace(line, search, replace):
    if search in line:
        return replace
    else:
        return line

def _md_word_replace(line, search, replace):
    if search in line:
        return line.replace(search, replace)
    else:
        return line

def _md_line_append(line, search, append):
    if search in line:
        return line + append
    else:
        return line

def make_deb():
    import subprocess
    from pynguin import conf
    import shutil
    from functools import partial

    CWD = os.getcwd()
    D = os.path.join(os.path.expanduser('~'), '__pynguin_debuild__')
    if os.path.exists(D):
        shutil.rmtree(D)
    os.mkdir(D)
    fn = '%s.tar.gz' % conf.version
    fpth = os.path.join(D, fn)
    tarX = os.path.join(CWD, '.tar-ignore')
    os.chdir('..')
    cmd = ['tar', 'zcf', fpth, '-X', tarX, 'pynguin']
    result = subprocess.run(cmd)
    result.check_returncode()

    os.chdir(D)
    cmd = ['py2dsc', '-m', 'Lee Harr <missive@hotmail.com>', fn]
    result = subprocess.run(cmd)
    result.check_returncode()

    os.chdir(os.path.join('deb_dist', conf.version))

    rules = [
        partial(_md_word_replace, search='unstable', replace='bionic'),
        ]
    _md_file_rewrite('debian/changelog', rules)

    rules = [
        partial(_md_line_replace, search='Package', replace='Package: pynguin'),
        partial(_md_line_append, search='Build-Depends', append=', python3 (>= 3.5)')
        ]
    _md_file_rewrite('debian/control', rules)

    f = open('debian/copyright', 'w')
    f.write('Copyright 2010-2018 Lee Harr <missive@hotmail.com>\n')
    f.close()

    append = [
        'override_dh_auto_install:',
            '\tpython3 setup.py install --root=debian/pynguin --install-layout=deb',
            '\trm -r debian/pynguin/usr/lib/python3/dist-packages/pynguin/__pycache__',
            '\trm -r debian/pynguin/usr/lib/python3/dist-packages/qsciedit/__pycache__',
        'override_dh_auto_build:',
        'override_dh_compress:',
            '\tdh_compress -X.pyn',
    ]
    _md_file_rewrite('debian/rules', append=append)

    shutil.copy('data/mimesupport/pynguin.sharedmimeinfo', 'debian')
    shutil.copy('data/mimesupport/pynguin.mime', 'debian')

    cmd = ['debuild']
    subprocess.run(cmd)

    cmd = ['debuild', '-S', '-sa']
    subprocess.run(cmd)

    # New steps for python3 deb build:
    # from http://wiki.debian.org/Python/Packaging
    #
    # Create tarball. Name as pynguin-X.Y.tar.gz
    #   $ tar zcf pynguin-X.Y.tar.gz -X pynguin/.tar-ignore  pynguin
    # Move it to a new workspace (new folder)
    #   $ py2dsc -m "Lee Harr <missive@hotmail.com>" pynguin-X.Y.tar.gz
    #   $ cd deb_dist/pynguin-X.Y
    # Modify debian/changelog
    #   # Change version "unstable" to "bionic" (or current release)
    # Modify debian/control
    #   # Change package name to pynguin
    #   # Add ", python3 (>= 3.6)" to Build-Depends
    # Add debian/copyright
    #   # Copyright 2010-2018 Lee Harr <missive@hotmail.com>
    # Modify debian/rules
    #   # Add lines:
    #       #override_dh_auto_install:
    #       #(tab) python3 setup.py install --root=debian/pynguin --install-layout=deb
    #       #(tab) rm -r debian/pynguin/usr/lib/python3/dist-packages/pynguin/__pycache__
    #       #(tab) rm -r debian/pynguin/usr/lib/python3/dist-packages/qsciedit/__pycache__
    #       #override_dh_auto_build:
    #       #override_dh_compress:
    #       #(tab) dh_compress -X.pyn
    # Copy MIME files
    #   # cp data/mimesupport/pynguin.sharedmimeinfo debian/
    #   # cp data/mimesupport/pynguin.mime debian/
    # For .deb:
    #   # debuild
    # For PPA:
    #   # debuild -S -sa
    #   # dput ppa:missive/ppa pynguin_X.Y-Z_source.changes

if '-d' in sys.argv:
    setup_devel()

    if 'MAKEDEB' in sys.argv:
        make_deb()
        sys.exit(0)


try:
    from pynguin import main
except ImportError:
    # No install found. Try to locate pynguin lib locally
    if '-d' not in sys.argv:
        sys.argv.append('-d')

    setup_devel()

    try:
        from pynguin import main
    except ImportError:
        print('Unable to locate the pynguin libraries.')
        print('Check your installation.')
        main = False
        raise


if main:
    if '-D' in sys.argv:
        main.setup_logging()

    from pynguin import util
    if not util.check_data_doc_dir():
        print('Unable to locate pynguin data.')
        print('Check your installation.')
        main = False

    if util.__file__.startswith('/usr/local') and sys.prefix == '/usr':
        print()
        print('If this is a Debian or Ubuntu system,')
        print('You may have forgotten to pass')
        print('--install-layout=deb')
        print('when installing.')


def help():
    from pynguin import conf
    print(conf.version)
    print()
    print('options:')
    print('  -h : this help text')
    print('  -d : development mode. Run from local directory')
    print('  -D : debug mode. Write debug info to log file')
    print(' DUMP <filename> :')
    print('       dump contents of pynguin file to standard output')
    print('  -T : run test suite')
    print('  -d MAKEDEB : create debian package')


pynfile = sys.argv[-1]
if not pynfile.endswith('.pyn'):
    pynfile = False


def run_tests():
    if '-d' not in sys.argv:
        sys.argv.append('-d')
        setup_devel()
        
    sys.argv = []
    
    import pytest
    pytest.main(['-v'])


if '-h' in sys.argv:
    help()
elif main and 'DUMP' in sys.argv:
    main.dumpfile()
elif '-T' in sys.argv:
    run_tests()
elif main and pynfile:
    main.run(pynfile)
elif main:
    main.run()
else:
    input()
